/*
    This file is part of the KDE project
    Copyright (C) 2011 Ernesto Rodriguez Ortiz <eortiz@uci.cu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "mmsthread.h"

MmsThread::MmsThread(const QString& url, const QString& name, int begin, int end)
: QThread(),
    m_sourceUrl(url),
    m_fileName(name),
    m_begin(begin),
    m_end(end),
    m_download(true)
{
    //Creating the file if it don't exist.
    fstream file;
    file.open(qstrdup(m_fileName.toAscii()), fstream::out | fstream::app);
    file.close();
}

void MmsThread::run()
{
    int read; // Seems that some times mmsx_read not read well.
    fstream file;
    mmsx_t* mms;
    
    //Reopening the file for write the information.
    file.open(qstrdup(m_fileName.toAscii()), fstream::in | fstream::out | fstream::ate);
    file.seekp(m_begin);
    
    //Connecting to the url
    mms = mmsx_connect(NULL, NULL, qstrdup(m_sourceUrl.toAscii()) , 1e6);
    mmsx_seek(0, mms, m_begin, 0);
    while ((m_begin != m_end) && m_download) {
        if ((m_begin + 1024) > m_end) { 
            const int var = m_end - m_begin;
            char data[var];
            read = mmsx_read(0, mms, data, var);
            m_locker.lock();
            emit reading(var, m_end, m_begin = m_end);
            if (read) {
                file.write(data, var);
            }
            m_locker.unlock();
            quit(); // NOTE: Keep "quit()" here, if not then the thread never finish.
        } else {
            char data[1024];
            read = mmsx_read(0, mms, data, 1024);
            m_locker.lock();
            emit reading(1024, m_end, m_begin += 1024);
            if (read) {
                file.write(data, 1024);
            }
            m_locker.unlock();
            
        }
    }
    
    file.close();
    mmsx_close(mms);
    exec();
}

void MmsThread::stop()
{
    m_download = false;
}
